<?php
/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    wishlist-dot-com-for-woocommerce
 * @subpackage wishlist-dot-com-for-woocommerce/public
 */

class WLDCFWC_Public {


	/**
	 * Common functionality used by wldcfwc_Public and wldcfwc_Admin
	 */
	use Wldcfwc_Trait;

	/**
	 * The ID of this plugin.
	 *
	 * @since  1.0.0
	 * @var    string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since  1.0.0
	 * @var    string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * KK The plugin options.
	 *
	 * @since  1.0.0
	 * @var    string    $options
	 */
	private $options;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since 1.0.0
	 * @param string $plugin_name The name of the plugin.
	 * @param string $version     The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;

		if ( ! isset( $this->options ) ) {
			$this->options = $this->wldcfwc_get_options();
		}
	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since 1.0.0
	 */
	public function wldcfwc_enqueue_styles() {

		$is_wc_page = $this->wldcfwc_is_woocommerce_page();
		if ( $is_wc_page ) {
			$handle = WLDCFWC_SLUG . '-public';

			// local file
			if ( WLDCFWC_USE_MIN_CSS ) {
				$ext = '.min.css';
			} else {
				$ext = '.css';
			}
			$css_path = WLDCFWC_URL . 'public/css/wldcfwc-public' . $ext;

			wp_enqueue_style( $handle, $css_path, array(), $this->version, 'all' );

			$inline_css = $this->wldcfwc_get_inline_css();
			$custom_css = $this->wldcfwc_get_custom_css( $this->options );
			$css        = $inline_css . $custom_css;
			wp_add_inline_style( $handle, $css );
		}
	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since 1.0.0
	 */
	public function wldcfwc_enqueue_scripts() {

		$is_wc_page = $this->wldcfwc_is_woocommerce_page();
		if ( $is_wc_page ) {

			// local file
			if ( WLDCFWC_USE_MIN_JS ) {
				$ext = '.min.js';
			} else {
				$ext = '.js';
			}

			// public, primary javascript
			$js_path       = WLDCFWC_URL . 'public/js/wldcfwc-add2wishlist' . $ext;
			$script_handle = WLDCFWC_SLUG . '_public';
			wp_enqueue_script( $script_handle, $js_path, array( 'jquery' ), $this->version, true );
			wp_localize_script(
				$script_handle,
				'wldcfwc_api_settings',
				array(
					'root'  => esc_url_raw( rest_url() ),
					'nonce' => wp_create_nonce( 'wp_rest' ),
				)
			);
			// add inline javascript that's set by the administrator in the admin panel
			if ( ! empty( $this->options['wishlist_custom_js'] ) ) {
				$custom_js = html_entity_decode( $this->options['wishlist_custom_js'], ENT_QUOTES, 'UTF-8' );
				wp_add_inline_script( $script_handle, $custom_js );
			}

			// url param is triggering the process of getting theme styles which is done via .js file below
			// nonce not required since we're not processing the $_GET variable, and there's no security implications
			if ( isset( $_GET['qs_flag_get_theme_style'] ) ) {
				// javascript used by admin panel to get theme styles
				$js_path       = WLDCFWC_URL . 'public/js/wldcfwc-get-theme-styles' . $ext;
				$script_handle = WLDCFWC_SLUG . '_get_theme_style_public';
				wp_enqueue_script( $script_handle, $js_path, array( 'jquery' ), $this->version, true );
			}
		}
	}

	/**
	 * Identify pages our plugin needs to interact with
	 *
	 * @return bool
	 */
	public function wldcfwc_is_woocommerce_page() {
		$is_wc_page = false;
		if ( ( is_woocommerce()
			|| is_cart()
			|| is_checkout()

			|| ( (int) get_option( WLDCFWC_WISHLIST_HOMEPAGE_PAGE_OPTION ) === get_the_ID() ) )
		) {
			$is_wc_page = true;
		}
		return $is_wc_page;
	}

	/**
	 * Define JS vars to be include in public facing pages so our JS can get option values and similar
	 * JS vars is inserted using wp_head hook
	 *
	 * @return void
	 */
	public function wldcfwc_global_js_var() {

		$is_wc_page = $this->wldcfwc_is_woocommerce_page( $this );

		if ( $is_wc_page ) {

			$global_js_key = WLDCFWC_SLUG . '_global_js';
			$global_js_val = get_option( $global_js_key );

			// all user inputted js vars are text strings and are escaped prior to saving to the database
			echo "<script type='text/javascript'>" . wp_kses_post( $global_js_val ) . '</script>';
		}
	}
	/**
	 * Register the Add to WishList.com Buttons for the public-facing side of the site.
	 *
	 * @since 1.0.0
	 */
	public function wldcfwc_add2WishList_wish_data( $product_id ) {

		// get the current post/product ID
		$current_product_id = $product_id;

		// get the product based on the ID
		$product = wc_get_product( $current_product_id );

		if ( $product ) {

			$product_data = $this->wldcfwc_get_product_data_wish( $product );

			$span_id       = 'wldcfwc_data_span_' . $current_product_id;
			$wish_data_esc = wc_esc_json( wp_json_encode( $product_data ) );

			$html = "<span id=\"$span_id\" style=\"display:none;\" class=\"wldcfwc-data-span\" data-wldcfwc-selected-prod-id=\"$current_product_id\" data-wldcfwc-wish=\"$wish_data_esc\"></span>";

			return $html;
		}
	}
	/**
	 * Get product data used on the shopping cart page for the Add to WishList button
	 *
	 * @param  $item
	 * @return void
	 */
	public function wldcfwc_add2WishList_wish_data_cart( $item ) {

		if ( ! empty( $item['variation_id'] ) ) {
			$current_product_id = $item['variation_id'];
		} else {
			$current_product_id = $item['product_id'];
		}

		// get the product based on the ID
		$product = wc_get_product( $current_product_id );

		if ( $product ) {
			$product_data = $this->wldcfwc_get_product_data_wish( $product );

			$wish_data_esc = wc_esc_json( wp_json_encode( $product_data ) );

			$span_id = 'wldcfwc_data_span_' . $current_product_id;
			echo '
                <span id="' . esc_attr( $span_id ) . '" style="display:none;" class="wldcfwc-data-span" data-wldcfwc-selected-prod-id="' . esc_attr( $current_product_id ) . '" data-wldcfwc-wish="' . esc_attr( $wish_data_esc ) . '"></span>
                ';
		}
	}
	/**
	 * Get product data used on the checkout page for the Add to WishList button
	 *
	 * @param  $item
	 * @return void
	 */
	public function wldcfwc_add2WishList_wish_data_checkout() {

		// Check if the cart is not empty
		if ( WC()->cart->is_empty() ) {
			return;
		}

		// Start with an empty array to hold our product IDs
		$product_ids = array();

		// Loop through all items in the cart
		$hidden_data_span = '';
		foreach ( WC()->cart->get_cart() as $cart_item_key => $item ) {
			// Get the product object
			$product = $item['data'];

			if ( ! empty( $item['variation_id'] ) ) {
				$current_product_id = $item['variation_id'];
			} else {
				$current_product_id = $item['product_id'];
			}

			// get the product based on the ID
			$product = wc_get_product( $current_product_id );

			if ( $product ) {
				$product_data = $this->wldcfwc_get_product_data_wish( $product );

				$wish_data_esc = wc_esc_json( wp_json_encode( $product_data ) );

				$span_id           = 'wldcfwc_data_span_' . $current_product_id;
				$hidden_data_span .= '
                    <span id="' . $span_id . '" style="display:none;" class="wldcfwc-data-span" data-wldcfwc-selected-prod-id="' . $current_product_id . '" data-wldcfwc-wish="' . $wish_data_esc . '"></span>
                    ';
			}
		}
		if ( ! empty( $hidden_data_span ) ) {
			$hidden_data_span = '<div style="display:hidden;" class="wldcfwc-data-span-parent">' . $hidden_data_span . '</div>';
			// no user inputed data to escape
			echo wp_kses_post( $hidden_data_span );
		}
	}
	/**
	 * Add a message above the checkout's shipping address
	 *
	 * @param  $item
	 * @return void
	 */
	public function wldcfwc_checkout_shipping_address_message_html() {
		$notice_html = "
            <ul id=\"wldcfwc_checkout_shipping_addresses_id\" class=\"woocommerce-info wldcfwc-hide\" role=\"alert\"></ul>
            <ul class=\"woocommerce-info wldcfwc-hide\" role=\"alert\"><li id='wldcfwc_different_shipping_addresses_warning_id'>Your cart might include gifts with different delivery addresses.</li></ul>
            ";

		echo wp_kses_post( $notice_html );
	}
	/**
	 * Add a message above the cart's  list of items
	 *
	 * @param  $item
	 * @return void
	 */
	public function wldcfwc_cart_shipping_address_message_html() {
		$notice_html = "
            <ul id=\"wldcfwc_cart_shipping_addresses_id\" class=\"woocommerce-info wldcfwc-hide\" role=\"alert\"></ul>
            <ul class=\"woocommerce-info wldcfwc-hide\" role=\"alert\"><li id='wldcfwc_different_shipping_addresses_warning_id'>Your cart might include gifts with different delivery addresses.</li></ul>
            ";

		echo wp_kses_post( $notice_html );
	}

	/**
	 * Get html for the Add to WishList button on the product page
	 *
	 * @return void
	 */
	public function wldcfwc_add2wishlist_button__product_page() {
		$mode           = 'product_page';
		$params['mode'] = $mode;

		global $product;
		if ( isset( $product ) ) {
			// if a shortcode is added to the wrong screen, the screen will no longer be editable if $product is not present for that screen
			$product_id                  = $product->get_id();
			$params['button_parameters'] = 'product_id=' . $product_id;
			$return_html                 = $this->wldcfwc_get_wishlist_button( $this->options, $params );

			// get hidden wish data
			$wish_data = $this->wldcfwc_add2WishList_wish_data( $product_id );
			// append hidden data to button.
			$return_html .= $wish_data;

		} else {
			// just in case
			$return_html = '';
		}

		$allowed_tags = wldcfwc_setAllowedTags( 'post,svg' );
		echo wp_kses( $return_html, $allowed_tags );
	}

	/**
	 * Get html for the Add to WishList button on the product page,
	 * when that button is positioned using JS
	 *
	 * @return void
	 */
	public function wldcfwc_add2wishlist_button__product_page_js_pos() {
		$mode                              = 'product_page';
		$params['mode']                    = $mode;
		$params['add_to_button_div_class'] = 'wldcfwc-hide';

		global $product;
		if ( isset( $product ) ) {
			// if a shortcode is added to the wrong screen, the screen will no longer be editable if $product is not present for that screen
			$product_id                  = $product->get_id();
			$params['button_parameters'] = 'product_id=' . $product_id;
			$return_html                 = $this->wldcfwc_get_wishlist_button( $this->options, $params );
		} else {
			// just in case
			$return_html = '';
		}

		$allowed_tags = wldcfwc_setAllowedTags( 'post,svg' );
		echo wp_kses( $return_html, $allowed_tags );
	}

	/**
	 * Get html for the Add to WishList button on the product loop
	 *
	 * @return void
	 */
	public function wldcfwc_add2wishlist_button__product_loop() {
		$mode           = 'product_loop';
		$params['mode'] = $mode;

		global $product;
		if ( isset( $product ) ) {
			// if a shortcode is added to the wrong screen, the screen will no longer be editable if $product is not present for that screen
			$product_id                  = $product->get_id();
			$params['button_parameters'] = 'product_id=' . $product_id;
			$return_html                 = $this->wldcfwc_get_wishlist_button( $this->options, $params );

			// get hidden wish data
			$wish_data = $this->wldcfwc_add2WishList_wish_data( $product_id );
			// append hidden data to button.
			$return_html .= $wish_data;

		} else {
			// just in case
			$return_html = '';
		}

		$allowed_tags = wldcfwc_setAllowedTags( 'post,svg' );
		echo wp_kses( $return_html, $allowed_tags );
	}

	/**
	 * Get html for the Add to WishList button on the product loop,
	 * when that button is positioned using JS
	 *
	 * @return void
	 */
	public function wldcfwc_add2wishlist_button__product_loop_js_pos() {
		$mode                              = 'product_loop';
		$params['mode']                    = $mode;
		$params['add_to_button_div_class'] = 'wldcfwc-hide';

		global $product;
		if ( isset( $product ) ) {
			// if a shortcode is added to the wrong screen, the screen will no longer be editable if $product is not present for that screen
			$product_id                  = $product->get_id();
			$params['button_parameters'] = 'product_id=' . $product_id;
			$return_html                 = $this->wldcfwc_get_wishlist_button( $this->options, $params );

			// get hidden wish data
			$wish_data = $this->wldcfwc_add2WishList_wish_data( $product_id );
			// append hidden data to button.
			$return_html .= $wish_data;

		} else {
			// just in case
			$return_html = '';
		}

		$allowed_tags = wldcfwc_setAllowedTags( 'post,svg' );
		echo wp_kses( $return_html, $allowed_tags );
	}

	/**
	 * Get html for the Add to WishList button on the cart
	 *
	 * @return void
	 */
	public function wldcfwc_add2wishlist_button__cart() {
		$atts_passed['mode'] = 'cart';
		$return_html         = $this->wldcfwc_get_add2wishlist_button_for_a_list( $this->options, $atts_passed );

		$allowed_tags = wldcfwc_setAllowedTags( 'post,svg' );
		echo wp_kses( $return_html, $allowed_tags );
	}

	/**
	 * Get html for the Add to WishList button on the cart,
	 * when that button is positioned using JS
	 *
	 * @return void
	 */
	public function wldcfwc_add2wishlist_button__cart_js_pos() {
		$atts_passed['mode']                    = 'cart';
		$atts_passed['add_to_button_div_class'] = 'wldcfwc-hide';
		$return_html                            = $this->wldcfwc_get_add2wishlist_button_for_a_list( $this->options, $atts_passed );

		$allowed_tags = wldcfwc_setAllowedTags( 'post,svg' );
		echo wp_kses( $return_html, $allowed_tags );
	}

	/**
	 * Get all the options
	 *
	 * @return array
	 */
	public function wldcfwc_return_options() {
		return $this->options;
	}

	/**
	 * Called when product page Add to WishList button is positioned using a shortcode
	 *
	 * @return string
	 */
	public function wldcfwc_product_page_button_shortcode() {

		$return_html = $this->wldcfwc_add2wishlist_button__product_page();
		return wp_kses( $return_html, $allowed_tags );
	}
	/**
	 * Called when product loop Add to WishList button is positioned using a shortcode
	 *
	 * @return string
	 */
	public function wldcfwc_poduct_loop_button_shortcode() {
		$return_html = $this->wldcfwc_add2wishlist_button__product_loop();

		$allowed_tags = wldcfwc_setAllowedTags( 'post,svg' );
		return wp_kses( $return_html, $allowed_tags );
	}
	/**
	 * Called when cart Add to WishList button is positioned using a shortcode
	 *
	 * @return string
	 */
	public function wldcfwc_cart_button_shortcode() {

		$atts_passed = array();

		// change container class so it's not positioned like other buttons
		$atts_passed['button_div_id']    = 'wldcfwc_save_all_wishlist_shortcode_button_div_id__cart';
		$atts_passed['button_div_class'] = 'wldcfwc-add2wishlist-button-short-code-div wldcfwc-list-button-div--show-hide';

		$atts_passed['wp-block-button__link--turnoff'] = true;

		$atts_passed['mode'] = 'cart';

		$return_html = $this->wldcfwc_get_add2wishlist_button_for_a_list( $this->options, $atts_passed );

		$allowed_tags = wldcfwc_setAllowedTags( 'post,svg' );
		return wp_kses( $return_html, $allowed_tags );
	}
	/**
	 * Used by the template page that displays the store's WishList homepage. Hooked by wldcfwc_hp_shortcode
	 *
	 * @return string
	 */
	public function wldcfwc_hp_shortcode() {
		$return_html = $this->wldcfwc_get_wishlist_hp_html();

		$allowed_tags = wldcfwc_setAllowedTags( 'post,svg' );
		return wp_kses( $return_html, $allowed_tags );
	}

	/**
	 * Used to remove the template page used to display WishList pages from the top menu
	 *
	 * @param  $args
	 * @param  $params
	 * @return mixed
	 */
	public function wldcfwc_remove_page_from_menu( $args, $params ) {
		$page_ids_a    = $params['remove_page_ids_a'];
		$page_ids_list = implode( ',', $page_ids_a );

		if ( ! isset( $args['exclude'] ) ) {
			$args['exclude'] = $page_ids_list;
		} else {
			$list            = $args['exclude'];
			$list_a          = explode( ',', $list );
			$list_a          = array_merge( $list_a, $page_ids_a );
			$list            = implode( ',', $list_a );
			$args['exclude'] = $list;
		}
		return $args;
	}

	/**
	 * Used to remove any WishList top menu items when the plugin is not set to live. Hooked with wp_page_menu_args
	 *
	 * @param  $args
	 * @return mixed
	 */
	public function wldcfwc_remove_all_wishlist_pages_from_menu( $args ) {
		// hook wp_page_menu_args()
		// update $args['exclude'] to exclude our template

		// This is only called when the plugin is active, but it's status is 'Testing' or 'Off'.
		// This removes the "WishList" link from the top menu for everyone when the plugin status is 'Off' and
		// for all non-admin users when the plugin status is 'Test'.
		$remove_page_id                = (int) get_option( WLDCFWC_WISHLIST_HOMEPAGE_PAGE_OPTION );
		$params['remove_page_ids_a'][] = $remove_page_id;

		$args = $this->wldcfwc_remove_page_from_menu( $args, $params );
		return $args;
	}

	/**
	 * Trigger call to WishList.com API asynchronously, via ajax call
	 * API call is get the user's WishList.com session data for items like a Wishe's delivery address for WooComm checkout page
	 *
	 * @return JSON
	 */
	public function wldcfwc_get_wlcom_user_session_rec_ajax() {

		if ( isset( $_POST['_wpnonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wpnonce'] ) ), 'wldcfwc_get_wlcom_user_session_rec_ajax' ) ) {

			// This is a key for the user's session on WishList.com. It's passed as a querstring param like
			// when a uses adds a wish to the store's cart where that wish is presented on WishList.com as part of the store's WishList
			if ( isset( $_POST['wldcom_user_session_id'] ) ) {
				$wldcom_user_session_id = sanitize_text_field( $_POST['wldcom_user_session_id'] );
			}

			if ( ! empty( $wldcom_user_session_id ) ) {
				// do api call to wldcom to get the use's session data
				$params['data']       = array( 'wldcom_user_session_id' => $wldcom_user_session_id );
				$params['api_action'] = 'get_user_session_data';
				$response             = wldcfwc_api_call( $params );
				wp_send_json( $response );
			}
		} else {
			die( 'Security check failed!' );
		}

		// Always exit to prevent extra output
		wp_die();
	}
}
